﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class UserInterfaceController : MonoBehaviour
{
    #region Constants

    public enum State
    {
        Intro,
        Placement,
        LaunchSoldiers,
        OutroFail,
        OutroWin,
        Hidden,
    }

    #endregion

    #region Fields

    public static UserInterfaceController Instance;

    public Canvas IntroUI;
    public Canvas PlacementUI;
    public Canvas LaunchSoldiersUI;
    public Canvas OutroFailUI;
    public Canvas OutroWinUI;

    public Text IntroTextArea;

    private State _currentState;

    private InventoryItemButton[] _inventoryItemButtons;

    #endregion

    #region UnityMethods

    // Use this for initialization
    void Awake()
    {
        Instance = this;
        _inventoryItemButtons = GetComponentsInChildren<InventoryItemButton>(true);
    }

    void Start()
    {
        
    }

    // Update is called once per frame
    void Update()
    {

    }

    #endregion

    #region Public Methods

    public void SetState(State newState)
    {
        _currentState = newState;

        switch (newState)
        {
        case State.Intro:
            IntroUI.gameObject.SetActive(true);
            PlacementUI.gameObject.SetActive(false);
            LaunchSoldiersUI.gameObject.SetActive(false);
            OutroFailUI.gameObject.SetActive(false);
            OutroWinUI.gameObject.SetActive(false);
            break;

        case State.Placement:
            IntroUI.gameObject.SetActive(false);
            PlacementUI.gameObject.SetActive(true);
            LaunchSoldiersUI.gameObject.SetActive(false);
            OutroFailUI.gameObject.SetActive(false);
            OutroWinUI.gameObject.SetActive(false);
            break;

        case State.LaunchSoldiers:
            IntroUI.gameObject.SetActive(false);
            PlacementUI.gameObject.SetActive(false);
            LaunchSoldiersUI.gameObject.SetActive(true);
            OutroFailUI.gameObject.SetActive(false);
            OutroWinUI.gameObject.SetActive(false);
            break;

        case State.OutroFail:
            IntroUI.gameObject.SetActive(false);
            PlacementUI.gameObject.SetActive(false);
            LaunchSoldiersUI.gameObject.SetActive(false);
            OutroFailUI.gameObject.SetActive(true);
            OutroWinUI.gameObject.SetActive(false);
            break;

        case State.OutroWin:
            IntroUI.gameObject.SetActive(false);
            PlacementUI.gameObject.SetActive(false);
            LaunchSoldiersUI.gameObject.SetActive(false);
            OutroFailUI.gameObject.SetActive(false);
            OutroWinUI.gameObject.SetActive(true);
            break;

        default:
            IntroUI.gameObject.SetActive(false);
            PlacementUI.gameObject.SetActive(false);
            LaunchSoldiersUI.gameObject.SetActive(false);
            OutroFailUI.gameObject.SetActive(false);
            OutroWinUI.gameObject.SetActive(false);
            break;
        }
    }

    public InventoryItemButton GetInventoryItemButton(int itemIndex)
    {
        return _inventoryItemButtons[itemIndex];
    }

    public void SetIntroText(string text)
    {
        if (IntroTextArea != null)
        {
            IntroTextArea.text = text;
        }
    }

    public void OnNextButtonPressed()
    {
        GameCycleManager.Instance.OnNextButtonPressed();
    }

    public void OnGoButtonPressed()
    {   
        GameCycleManager.Instance.OnGoButtonPressed();
    }

    public void OnAbortButtonPressed()
    {
        GameCycleManager.Instance.OnAbortButtonPressed();
    }

    public void OnInventoryButtonPressed(int itemIndex)
    {
        GameCycleManager.Instance.OnInventoryButtonPressed(itemIndex);
    }

    public void OnContinueButtonPressed()
    {
        GameCycleManager.Instance.OnContinueButtonPressed();
    }

    public void OnRetryButtonPressed()
    {
        GameCycleManager.Instance.OnRetryButtonPressed();
    }

    public void OnQuitButtonPressed()
    {
        GameCycleManager.Instance.OnQuitButtonPressed();
    }

    #endregion
}
