﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Collider2D))]
public class BaseCollisionDamageProvider : MonoBehaviour
{
    public struct CollisionDamageInfo
    {
        public float Damage;
        public bool Kill;
    }

    #region Fields

    public float MaxHealthDelta = -1;

    protected Collider2D Collider;

    #endregion

    #region Unity Methods

    // Use this for initialization
    public virtual void Awake()
    {
        Collider = GetComponent<Collider2D>();
    }

    #endregion

    #region Public Methods

    /// <summary>
    /// Handles a collision with a damageable object.
    /// </summary>
    /// <param name="impactVelocitySq">The velocity of the collision squared</param>
    /// <param name="targetObject">The colliding object</param>
    /// <returns>The Damage to apply on from collision</returns>
    public virtual CollisionDamageInfo GetCollisionDamage(float impactVelocitySq, GameObject targetObject)
    {
        return new CollisionDamageInfo() {Damage = MaxHealthDelta};
    }

    #endregion
}
