﻿// #define DEBUG_HEALTH

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class HealthComponent : MonoBehaviour
{
    public delegate void HealthEmptyCallBack();

    #region Fields

    public float MaxHealth;
    
    public HealthEmptyCallBack EmptyCallBack;
    public float CurrentHealth;

    public SpriteRenderer DebugSprite;

#endregion

    #region UnityMethods

    public void OnEnable()
    {
        CurrentHealth = MaxHealth;

#if DEBUG_HEALTH
        UpdateDebug();
#endif
    }

    #endregion

    #region Public Methods

    public void AddHealth(float amount)
    {
        CurrentHealth = Mathf.Min( CurrentHealth + amount, MaxHealth);

        if (CurrentHealth <= 0)
        {
            CurrentHealth = 0;

            if (EmptyCallBack != null)
            {
                EmptyCallBack.Invoke();
            }
        }

#if DEBUG_HEALTH
        UpdateDebug();
#endif
    }

    #endregion

#if DEBUG_HEALTH
    private void UpdateDebug()
    {
        if (DebugSprite != null)
        { 
            var delta = Mathf.InverseLerp(0f, MaxHealth, CurrentHealth);

            if (delta < 0.5f)
            {
                DebugSprite.color = Color.Lerp(Color.red, Color.yellow, delta * 2);
            }
            else
            {
                DebugSprite.color = Color.Lerp(Color.yellow, Color.green, (delta - 0.5f) * 2);
            }

            
        }
    }
#endif
}
