﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Collider2D))]
public class SoldierKillCollisionDamageProvider : BaseCollisionDamageProvider
{
    public delegate void KillCountFullCallBack();

    #region Fields

    public int MaxKillCount;
    public int InitialKillCount;

    public KillCountFullCallBack FullCallBack;

    #endregion

    #region Properties

    public int CurrentKillCount { get; private set; }

    #endregion

    #region Unity Methods

    // Use this for initialization
    public override void Awake()
    {
        base.Awake();
        CurrentKillCount = InitialKillCount;
//        Reset();
    }

    #endregion

    #region Public Methods

    /// <summary>
    /// Handles a collision with a damageable object.
    /// </summary>
    /// <param name="impactVelocitySq">The velocity of the collision squared</param>
    /// <param name="targetObject">The colliding object</param>
    /// <returns>The Damage to apply on from collision</returns>
    public override CollisionDamageInfo GetCollisionDamage(float impactVelocitySq, GameObject targetObject)
    {
        var kill = false;

        if (CurrentKillCount < MaxKillCount && targetObject.GetComponent<BaseSoldier>() != null)
        {
            CurrentKillCount++;
            kill = true;

            if (CurrentKillCount == MaxKillCount)
            {
                FullCallBack();
            }
        }
        
        return new CollisionDamageInfo() {Damage = MaxHealthDelta, Kill = kill };
    }

//    public void Reset()
//    {
//        CurrentKillCount = InitialKillCount;
//    }

    #endregion
}
