﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Collider2D))]
public class VelocityCollisionDamageProvider : BaseCollisionDamageProvider
{
    #region Fields

    public float MinHealthDelta = 0f;

    public float MinSpeed = 20f;
    public float MaxSpeed = 50f;

    protected float MinSpeedSq;
    protected float MaxSpeedSq;

    #endregion

    #region Unity Methods

    // Use this for initialization
    public override void Awake()
    {
        base.Awake();

        MinSpeedSq = MinSpeed * MinSpeed;
        MaxSpeedSq = MaxSpeed * MaxSpeed;
    }

    #endregion

    #region Public Methods

    /// <summary>
    /// Handles a collision with a damageable object.
    /// </summary>
    /// <param name="impactVelocitySq">The velocity of the collision squared</param>
    /// <param name="targetObject">The colliding object</param>
    /// <returns>The Damage to apply on from collision</returns>
    public override CollisionDamageInfo GetCollisionDamage(float impactVelocitySq, GameObject targetObject)
    {
        float damageScalar = Mathf.InverseLerp(MinSpeedSq, MaxSpeedSq, impactVelocitySq);
        float damage = Mathf.Lerp(MinHealthDelta, MaxHealthDelta, damageScalar);

        return new CollisionDamageInfo() {Damage = damage};
    }

    #endregion
}
