﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(HealthComponent), typeof(SoldierDeathComponent), typeof(Rigidbody2D))]
public class BaseSoldier : MonoBehaviour
{
    #region Fields

    public GameObject Body;
    public GameObject RearArm;
    public GameObject FrontArm;
    public float BodyRotationRate = 0.1f;
    public float MinSpeedForRotationUpdate = 2f;

    protected HealthComponent Health;
    protected SoldierDeathComponent Death;
    protected Rigidbody2D RigidBody;
    protected float LastFrameAngle;
    protected Quaternion LastFrameBodyRotation;

    #endregion

    #region Properties

    public SpawnPointBase SpawnPoint { set; get; }

    #endregion

    #region Unity Methods

    // Use this for initialization
    void Awake()
    {
        Health = GetComponent<HealthComponent>();
        Death = GetComponent<SoldierDeathComponent>();
        RigidBody = GetComponent<Rigidbody2D>();

        Death.soldier = this;
    }

    void OnEnable()
    {
        LastFrameAngle = 0f;
        LastFrameBodyRotation = Quaternion.identity;
    }

    void Update()
    {
        if (RigidBody.velocity.sqrMagnitude > MinSpeedForRotationUpdate * MinSpeedForRotationUpdate)
        {
            Vector2 dir = RigidBody.velocity;
            float angle = (Mathf.Atan2(dir.y, dir.x) * Mathf.Rad2Deg) + 180f;
            Quaternion rotation = Quaternion.AngleAxis(angle, Vector3.forward);

            LastFrameBodyRotation = Quaternion.Lerp(LastFrameBodyRotation, rotation, BodyRotationRate);
        }

        LastFrameAngle = LastFrameBodyRotation.eulerAngles.z;

        if (LastFrameAngle > 90f && LastFrameAngle < 270f)
        {
            Body.transform.localScale = new Vector3(1f, -1f, 1f);
        }
        else
        {
            Body.transform.localScale = new Vector3(1f, 1f, 1f);
        }

        Body.transform.rotation = LastFrameBodyRotation;

        RearArm.transform.rotation = Quaternion.AngleAxis(RigidBody.rotation, Vector3.forward);
        FrontArm.transform.rotation = Quaternion.AngleAxis(RigidBody.rotation + 180f, Vector3.forward);
    }

    #endregion

    #region Public Methods

    public void Activate(Vector3 position, Quaternion rotation, float velocity)
    {
        gameObject.transform.position = position;
        gameObject.SetActive(true);
        RigidBody.velocity = (rotation * new Vector2(-velocity, 0f));
        RigidBody.angularVelocity = UnityEngine.Random.Range(-540f, 540f);
    }

    public void Despawn()
    {
        SpawnPoint.DespawnSoldier(this);
    }

    #endregion
}
