﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public abstract class SpawnPointBase : MonoBehaviour
{
    //public delegate BaseSoldier GetSoldierFromPool();
    //public delegate void ReturnSoldierToPool(BaseSoldier soldier);

    public int TotalSoldierCount = 100;
    public BaseSoldier SoldierPrefab;
    public bool AddDespawnedSoldiersToAvailableCount;

    private LinkedList<BaseSoldier> _disabledSoldierPool;
    private LinkedList<BaseSoldier> _enabledSoldierPool;

    #region Structs

    #endregion

    #region Fields

    //public GetSoldierFromPool SpawnSoldierCallBack;
    //public ReturnSoldierToPool DespawnSoldierCallBack;

    protected int RemainingSoldiers = -1;

    #endregion

    #region Properties

    public int ActiveSoldierCount { get; private set; }
    public int NonDeadSoldierCount { get { return ActiveSoldierCount + RemainingSoldiers; } }

    #endregion

    #region UnityMethods

    void Awake()
    {
        _disabledSoldierPool = new LinkedList<BaseSoldier>();
        _enabledSoldierPool = new LinkedList<BaseSoldier>();
    }

    void Start()
    {
        if (RemainingSoldiers != -1)
        {
            Debug.Break();
        }

        RemainingSoldiers = TotalSoldierCount;

        for (int i = 0; i < TotalSoldierCount; i++)
        {
            GameObject newSoldier = Instantiate(SoldierPrefab.gameObject);
            newSoldier.SetActive(false);
            var newSoldierComponent = newSoldier.GetComponent<BaseSoldier>();
            newSoldierComponent.SpawnPoint = this;

            _disabledSoldierPool.AddLast(newSoldierComponent);
        }
    }

    #endregion

    #region Public Methods

    public abstract void StartSpawn();

    public BaseSoldier SpawnSoldier()
    {
        if (RemainingSoldiers <= 0)
        {
            Debug.Log(name + " -> Unable to spawn soldier from pool.");
            return null;
        }

        RemainingSoldiers--;
        ActiveSoldierCount++;

        var soldier = _disabledSoldierPool.First.Value;
        _disabledSoldierPool.RemoveFirst();
        _enabledSoldierPool.AddLast(soldier);

        return soldier;
    }

    public void DespawnSoldier(BaseSoldier soldier)
    {
        var soldierNode = _enabledSoldierPool.Find(soldier);

        if (soldierNode != null)
        {
            soldier.gameObject.SetActive(false);
            _enabledSoldierPool.Remove(soldierNode);
            _disabledSoldierPool.AddLast(soldier);

            ActiveSoldierCount--;

            if (AddDespawnedSoldiersToAvailableCount)
            {
                RemainingSoldiers++;
            }
        }
    }

    #endregion
}
